@@ -1,11 +1,12 @@
 from pathlib import Path
 import pandas as pd
 from sequana import logger
+import re
 
 logger.name = __name__
 
 
-def get_most_probable_strand(sample_folder, tolerance=0.10):
+def get_most_probable_strand(sample_folder, tolerance):
     """Return total counts by strand from featureCount matrix folder, strandness and
     probable strand for a single sample (using a tolerance threshold for
     strandness). This assumes a single sample by featureCounts file.
@@ -31,16 +32,14 @@ def get_most_probable_strand(sample_folder, tolerance=0.10):
     elif 0.5 - tolerance < strandness and strandness < 0.5 + tolerance:
         res_dict["strand"] = "0"
     else:
-        raise IOError(
-            f"No strandness could be inferred from the count files for '{sample_name}' with a tolerance of {tolerance}. Value of 'strandness': {strandness:.2f}"
-        )
+        res_dict["strand"] = "NA"
 
     df = pd.DataFrame(res_dict, index=[sample_name])
 
     return df
 
 
-def get_most_probable_strand_consensus(rnaseq_folder):
+def get_most_probable_strand_consensus(rnaseq_folder, tolerance):
     """From a sequana rna-seq run folder get the most probable strand, based on the
     frequecies of counts assigned with '0', '1' or '2' type strandness
     (featureCounts nomenclature)
@@ -52,10 +51,13 @@ def get_most_probable_strand_consensus(rnaseq_folder):
     )
 
     df = pd.concat(
-        [get_most_probable_strand(sample_folder) for sample_folder in sample_folders]
+        [
+            get_most_probable_strand(sample_folder, tolerance)
+            for sample_folder in sample_folders
+        ]
     )
 
-    logger.info("Strandness probability report:")
+    logger.info("Strand guessing for each files (tolerance: {tolerance}):\n {df}")
     logger.info(df)
 
     probable_strands = df.loc[:, "strand"].unique()
@@ -64,12 +66,16 @@ def get_most_probable_strand_consensus(rnaseq_folder):
         return probable_strands[0]
     else:
         raise IOError(
-            f"No consensus on most probable strand. Could be: {probable_strands}"
+            f"No consensus on most probable strand found with a tolerance of {tolerance}. \
+Could be: {' or '.join(str(x) for x in probable_strands)}."
         )
 
 
 class FeatureCount:
-    """ Read a featureCounts output file.
+    """Read a featureCounts output file.
+
+    #TODO: Test on single and multi featurecounts files and see how self.df and
+    self.rnadiff_ready_df behave.
     """
 
     def __init__(
@@ -78,12 +84,13 @@ class FeatureCount:
         clean_sample_names=True,
         extra_name_rm=["_Aligned"],
         drop_loc=True,
+        rnadiff_ready=True,
     ):
         """.. rubric:: Constructor
 
         Get the featureCounts output as a pandas DataFrame
         :param bool clean_sample_names: if simplifying the sample names in featureCount output columns
-        - extra_name_rm: extra list of strings to remove from samples_names (ignored if clean_sample_name is False)
+        - extra_name_rm: extra list of regex to remove from samples_names (ignored if clean_sample_name is False)
         - drop_loc: if dropping the extrac location columns (ie getting only the count matrix)
         """
 
@@ -95,19 +102,49 @@ class FeatureCount:
         self.extra_name_rm = extra_name_rm
         self.drop_loc = drop_loc
         self._df = self._get_df()
+        # Count matrix prepared for rnadiff
+        if rnadiff_ready:
+            self.rnadiff_ready_df = self._get_rnadiff_ready_df()
+            self.target_df = self._get_target_df()
+
+    def _get_rnadiff_ready_df(self):
+        """Prepare a count matrix from a multi-sample featureCount file.
+
+        This is in particular add a 'rawCounts_' to column names to not have an
+        import problem in R with labels starting with numbers
+        """
+        df = pd.read_csv(self.filename, sep="\t", comment="#", index_col=0)
+        df.drop(["Chr", "Start", "End", "Strand", "Length"], axis=1, inplace=True)
+        df.columns = [
+            self._clean_sample_names(x, self.extra_name_rm) for x in df.columns
+        ]
+        df.set_index(df.columns[0], inplace=True)
+        return df
+
+    def _get_target_df(self):
+        """ Prepare the table with grouping information for rnadiff
+        """
+        df = pd.DataFrame(
+            {
+                "sample": self.rnadiff_ready_df.columns,
+                "label": self.rnadiff_ready_df.columns.str.replace("rawCounts_", ""),
+                "condition": None,
+            }
+        )
+        df.set_index(df.columns[0], inplace=True)
+        return df
 
     def _get_df(self):
 
         df = pd.read_csv(self.filename, sep="\t", comment="#", index_col=0)
 
+        if self.drop_loc:
+            df.drop(["Chr", "Start", "End", "Strand", "Length"], axis=1, inplace=True)
+
         if self.clean_sample_names:
             df.columns = [
                 self._clean_sample_names(x, self.extra_name_rm) for x in df.columns
             ]
-
-        if self.drop_loc:
-            df.drop(["Chr", "Start", "End", "Strand", "Length"], axis=1, inplace=True)
-
         return df
 
     df = property(_get_df)
@@ -119,7 +156,7 @@ class FeatureCount:
         new_name = new_name.split(".")[0]
 
         for pattern in extra_name_rm:
-            new_name = new_name.replace(pattern, "")
+            new_name = re.sub(pattern, "", new_name)
 
         return new_name
 
